require 'yamd'
require 'addressable/uri'

class MangafoxPage < PageCrawler
  def image_url
    @parsed_html.at_css('#viewer a img')['src']
  end
end

class MangafoxChapter < ChapterCrawler
  def self.page_class
    MangafoxPage
  end

  def pages_info
    # there's no need of an lazy enumerator here, no IO action is taken
    page_options = @parsed_html.at_css('.prev_page + div > select').css('option')
    pages_number = page_options.map { | option | option['value'].to_i }.max
    
    chapter_base_url = Addressable::URI.parse(self.url)
    chapter_base_path, _ = Pathname.new(chapter_base_url.path).split
    page_urls = (1..pages_number).to_a.map do | i |
      chapter_base_url.path = chapter_base_path.join("#{i}.html").to_s
      { url: chapter_base_url.to_s }
    end

    page_urls
  end

  def name
    @custom_data[:name]
  end
end

class MangafoxCrawler < MangaCrawler
  def chapters_info
    @parsed_html.css('#chapters ul li div').reverse.map do | chapter_link |
      name = chapter_link.at_css('a.tips').text
      # the real chapter name not always exist
      span_with_real_name = chapter_link.at_css('a.tips + span')
      name = name + ' - ' + span_with_real_name.text if span_with_real_name
      { name: name,
        url: chapter_link.at_css('a.tips')['href']
      }
    end
  end

  def self.chapter_class
    MangafoxChapter
  end

  def name
    @parsed_html.at_css('h1').text.match(/(.+) Manga/)[1]
  end
end

